import { env } from '$env/dynamic/private';
import { csrf } from '$lib/server/csrf';
import { DB } from '$lib/server/drizzle';
import { runSeeds } from '$lib/server/drizzle/seeds';
import { JWT } from '$lib/server/jwt';
import type { ThemeModeType } from '$lib/theme-mode';
import type { Handle } from '@sveltejs/kit';
import { sequence } from '@sveltejs/kit/hooks';
import { migrate } from 'drizzle-orm/mysql2/migrator';
import { handleSession } from 'svelte-kit-cookie-session';

const { AUTO_MIGRATE, SESSION_SECRET, SESSION_SECURE } = env;

// Initialize the database
await DB.init();

// Initialize the JWT keys
await JWT.init();

// Migrate the database when automatic migration is enabled
if (AUTO_MIGRATE === 'true') {
	await migrate(DB.drizzle, { migrationsFolder: './migrations' });
}

// Run database data seeders
await runSeeds();

/**
 * Set the document theme mode from the cookies
 */
const handleThemeHook = (async ({ resolve, event }) => {
	// Take the theme from the cookies
	const cookieTheme = event.cookies.get('themeMode') as ThemeModeType;
	if (cookieTheme) {
		return await resolve(event, {
			transformPageChunk: ({ html }) => html.replace('theme-base=""', `theme-base="${cookieTheme}"`)
		});
	}

	return await resolve(event);
}) satisfies Handle;

export const handle = sequence(
	csrf(['/oauth2/token', '/oauth2/introspect', '/oauth2/device_authorization']),
	handleSession({
		secret: SESSION_SECRET,
		cookie: {
			secure: SESSION_SECURE === 'true'
		}
	}),
	handleThemeHook
);
